@@ -202,6 +202,7 @@ void QalculateQtSettings::loadPreferences() {
 	evalops.interval_calculation = INTERVAL_CALCULATION_VARIANCE_FORMULA;
 
 	title_type = TITLE_APP;
+	auto_calculate = true;
 	dot_question_asked = false;
 	implicit_question_asked = false;
 	complex_angle_form = false;
@@ -255,6 +256,10 @@ void QalculateQtSettings::loadPreferences() {
 #endif
 	last_version_check_date.setToCurrentDate();
 
+	favourite_functions_changed = false;
+	favourite_variables_changed = false;
+	favourite_units_changed = false;
+
 	FILE *file = NULL;
 	std::string filename = buildPath(getLocalDir(), "qalculate-qt.cfg");
 	file = fopen(filename.c_str(), "r");
@@ -276,7 +281,9 @@ void QalculateQtSettings::loadPreferences() {
 	default_plot_use_sampling_rate = true;
 	max_plot_time = 5;
 
-	int version_numbers[] = {3, 21, 0};
+	preferences_version[0] = 3;
+	preferences_version[1] = 22;
+	preferences_version[2] = 0;
 
 	if(file) {
 		char line[1000000L];
@@ -312,8 +319,17 @@ void QalculateQtSettings::loadPreferences() {
 					if(!v_exact.empty()) v_exact[settings->v_exact.size() - 1].push_back(v);
 				} else if(svar == "expression_history") {
 					expression_history.push_back(svalue);
+				} else if(svar == "favourite_function") {
+					favourite_functions_pre.push_back(svalue);
+					favourite_functions_changed = true;
+				} else if(svar == "favourite_unit") {
+					favourite_units_pre.push_back(svalue);
+					favourite_units_changed = true;
+				} else if(svar == "favourite_variable") {
+					favourite_variables_pre.push_back(svalue);
+					favourite_variables_changed = true;
 				} else if(svar == "version") {
-					parse_qalculate_version(svalue, version_numbers);
+					parse_qalculate_version(svalue, preferences_version);
 				} else if(svar == "always_on_top") {
 					always_on_top = v;
 				} else if(svar == "keep_function_dialog_open") {
@@ -338,6 +354,24 @@ void QalculateQtSettings::loadPreferences() {
 					functions_vsplitter_state = QByteArray::fromBase64(svalue.c_str());
 				} else if(svar == "functions_hsplitter_state") {
 					functions_hsplitter_state = QByteArray::fromBase64(svalue.c_str());
+				} else if(svar == "units_geometry") {
+					units_geometry = QByteArray::fromBase64(svalue.c_str());
+				} else if(svar == "units_vsplitter_state") {
+					units_vsplitter_state = QByteArray::fromBase64(svalue.c_str());
+				} else if(svar == "units_hsplitter_state") {
+					units_hsplitter_state = QByteArray::fromBase64(svalue.c_str());
+				} else if(svar == "variables_geometry") {
+					variables_geometry = QByteArray::fromBase64(svalue.c_str());
+				} else if(svar == "variables_vsplitter_state") {
+					variables_vsplitter_state = QByteArray::fromBase64(svalue.c_str());
+				} else if(svar == "variables_hsplitter_state") {
+					variables_hsplitter_state = QByteArray::fromBase64(svalue.c_str());
+				} else if(svar == "datasets_geometry") {
+					datasets_geometry = QByteArray::fromBase64(svalue.c_str());
+				} else if(svar == "datasets_vsplitter_state") {
+					datasets_vsplitter_state = QByteArray::fromBase64(svalue.c_str());
+				} else if(svar == "datasets_hsplitter_state") {
+					datasets_hsplitter_state = QByteArray::fromBase64(svalue.c_str());
 				} else if(svar == "style") {
 					style = v;
 				} else if(svar == "palette") {
@@ -602,6 +636,8 @@ void QalculateQtSettings::loadPreferences() {
 					if(v >= INTERVAL_CALCULATION_NONE && v <= INTERVAL_CALCULATION_SIMPLE_INTERVAL_ARITHMETIC) {
 						evalops.interval_calculation = (IntervalCalculation) v;
 					}
+				} else if(svar == "calculate_as_you_type") {
+					auto_calculate = v;
 				} else if(svar == "chain_mode") {
 					chain_mode = v;
 				} else if(svar == "rpn_mode") {
@@ -711,6 +747,50 @@ void QalculateQtSettings::loadPreferences() {
 	else if(palette >= 0) updatePalette();
 
 }
+void QalculateQtSettings::updateFavourites() {
+	if(favourite_functions_pre.empty()) {
+		for(size_t i = 0; i < CALCULATOR->functions.size(); i++) {
+			if(CALCULATOR->functions[i]->isLocal() && !CALCULATOR->functions[i]->isHidden()) favourite_functions.push_back(CALCULATOR->functions[i]);
+		}
+	} else {
+		for(size_t i = 0; i < favourite_functions_pre.size(); i++) {
+			MathFunction *f = CALCULATOR->getActiveFunction(favourite_functions_pre[i]);
+			if(f) favourite_functions.push_back(f);
+		}
+	}
+	if(favourite_units_pre.empty()) {
+		const char *si_units[] = {"m", "kg_c", "s", "A", "K", "mol", "cd"};
+		for(size_t i = 0; i < 7; i++) {
+			Unit *u = CALCULATOR->getActiveUnit(si_units[i]);
+			if(!u) u = CALCULATOR->getCompositeUnit(si_units[i]);
+			if(u) favourite_units.push_back(u);
+		}
+		for(size_t i = 0; i < CALCULATOR->units.size(); i++) {
+			if(CALCULATOR->units[i]->isLocal() && !CALCULATOR->units[i]->isHidden()) favourite_units.push_back(CALCULATOR->units[i]);
+		}
+	} else {
+		for(size_t i = 0; i < favourite_units_pre.size(); i++) {
+			Unit *u = CALCULATOR->getActiveUnit(favourite_units_pre[i]);
+			if(!u) u = CALCULATOR->getCompositeUnit(favourite_units_pre[i]);
+			if(u) favourite_units.push_back(u);
+		}
+	}
+	if(favourite_variables_pre.empty()) {
+		favourite_variables.push_back(CALCULATOR->getVariableById(VARIABLE_ID_PI));
+		favourite_variables.push_back(CALCULATOR->getVariableById(VARIABLE_ID_E));
+		favourite_variables.push_back(CALCULATOR->getVariableById(VARIABLE_ID_EULER));
+		Variable *v = CALCULATOR->getActiveVariable("golden");
+		if(v) favourite_variables.push_back(v);
+		for(size_t i = 0; i < CALCULATOR->variables.size(); i++) {
+			if(CALCULATOR->variables[i]->isLocal() && !CALCULATOR->variables[i]->isHidden()) favourite_variables.push_back(CALCULATOR->variables[i]);
+		}
+	} else {
+		for(size_t i = 0; i < favourite_variables_pre.size(); i++) {
+			Variable *v = CALCULATOR->getActiveVariable(favourite_variables_pre[i]);
+			if(v) favourite_variables.push_back(v);
+		}
+	}
+}
 void QalculateQtSettings::updatePalette() {
 	QPalette p;
 	if(palette == 1) {
@@ -786,8 +866,8 @@ void QalculateQtSettings::savePreferences(bool) {
 		fprintf(file, "last_version_check=%s\n", last_version_check_date.toISOString().c_str());
 		if(!last_found_version.empty()) fprintf(file, "last_found_version=%s\n", last_found_version.c_str());
 	}
-	fprintf(file, "window_state=%s\n", window_state.toBase64().data());
-	fprintf(file, "window_geometry=%s\n", window_geometry.toBase64().data());
+	if(!window_state.isEmpty()) fprintf(file, "window_state=%s\n", window_state.toBase64().data());
+	if(!window_geometry.isEmpty()) fprintf(file, "window_geometry=%s\n", window_geometry.toBase64().data());
 	if(rpn_shown) fprintf(file, "rpn_shown=%i\n", rpn_shown);
 	fprintf(file, "splitter_state=%s\n", splitter_state.toBase64().data());
 	if(!functions_geometry.isEmpty()) fprintf(file, "functions_geometry=%s\n", functions_geometry.toBase64().data());
@@ -800,6 +880,9 @@ void QalculateQtSettings::savePreferences(bool) {
 	if(!variables_geometry.isEmpty()) fprintf(file, "variables_geometry=%s\n", variables_geometry.toBase64().data());
 	if(!variables_vsplitter_state.isEmpty()) fprintf(file, "variables_vsplitter_state=%s\n", variables_vsplitter_state.toBase64().data());
 	if(!variables_hsplitter_state.isEmpty()) fprintf(file, "variables_hsplitter_state=%s\n", variables_hsplitter_state.toBase64().data());
+	if(!datasets_geometry.isEmpty()) fprintf(file, "datasets_geometry=%s\n", datasets_geometry.toBase64().data());
+	if(!datasets_vsplitter_state.isEmpty()) fprintf(file, "datasets_vsplitter_state=%s\n", datasets_vsplitter_state.toBase64().data());
+	if(!datasets_hsplitter_state.isEmpty()) fprintf(file, "datasets_hsplitter_state=%s\n", datasets_hsplitter_state.toBase64().data());
 	fprintf(file, "always_on_top=%i\n", always_on_top);
 	if(title_type != TITLE_APP) fprintf(file, "window_title_mode=%i\n", title_type);
 	fprintf(file, "save_mode_on_exit=%i\n", save_mode_on_exit);
@@ -902,6 +985,7 @@ void QalculateQtSettings::savePreferences(bool) {
 	else if(dual_approximation > 0) fprintf(file, "approximation=%i\n", APPROXIMATION_APPROXIMATE + 1);
 	else fprintf(file, "approximation=%i\n", evalops.approximation);
 	fprintf(file, "interval_calculation=%i\n", evalops.interval_calculation);
+	fprintf(file, "calculate_as_you_type=%i\n", auto_calculate);
 	fprintf(file, "rpn_mode=%i\n", rpn_mode);
 	fprintf(file, "chain_mode=%i\n", chain_mode);
 	fprintf(file, "limit_implicit_multiplication=%i\n", evalops.parse_options.limit_implicit_multiplication);
@@ -975,15 +1059,36 @@ void QalculateQtSettings::savePreferences(bool) {
 			gsub("\n", " ", expression_history[i]);
 			fprintf(file, "expression_history=%s\n", expression_history[i].c_str());
 		}
+		if(favourite_functions_changed) {
+			for(size_t i = 0; i < favourite_functions.size(); i++) {
+				if(CALCULATOR->stillHasFunction(favourite_functions[i]) && favourite_functions[i]->isActive() && favourite_functions[i]->category() != CALCULATOR->temporaryCategory()) {
+					fprintf(file, "favourite_function=%s\n", favourite_functions[i]->referenceName().c_str());
+				}
+			}
+		}
+		if(favourite_units_changed) {
+			for(size_t i = 0; i < favourite_units.size(); i++) {
+				if(CALCULATOR->stillHasUnit(favourite_units[i]) && favourite_units[i]->isActive() && favourite_units[i]->category() != CALCULATOR->temporaryCategory()) {
+					fprintf(file, "favourite_unit=%s\n", favourite_units[i]->referenceName().c_str());
+				}
+			}
+		}
+		if(favourite_variables_changed) {
+			for(size_t i = 0; i < favourite_variables.size(); i++) {
+				if(CALCULATOR->stillHasVariable(favourite_variables[i]) && favourite_variables[i]->isActive() && favourite_variables[i]->category() != CALCULATOR->temporaryCategory()) {
+					fprintf(file, "favourite_variable=%s\n", favourite_variables[i]->referenceName().c_str());
+				}
+			}
+		}
 	}
 	fclose(file);
 
 }
 
-const char *QalculateQtSettings::multiplicationSign() {
+const char *QalculateQtSettings::multiplicationSign(bool units) {
 	if(!printops.use_unicode_signs) return "*";
 	switch(printops.multiplication_sign) {
-		case MULTIPLICATION_SIGN_X: {return SIGN_MULTIPLICATION;}
+		case MULTIPLICATION_SIGN_X: {if(units) {return SIGN_MIDDLEDOT;} return SIGN_MULTIPLICATION;}
 		case MULTIPLICATION_SIGN_DOT: {return SIGN_MULTIDOT;}
 		case MULTIPLICATION_SIGN_ALTDOT: {return SIGN_MIDDLEDOT;}
 		default: {return "*";}
@@ -995,8 +1100,25 @@ const char *QalculateQtSettings::divisionSign(bool output) {
 	return "/";
 }
 
+std::string QalculateQtSettings::localizeExpression(std::string str, bool unit_expression) {
+	ParseOptions pa = evalops.parse_options; pa.base = 10;
+	str = CALCULATOR->localizeExpression(str, pa);
+	gsub("*", multiplicationSign(unit_expression), str);
+	gsub("/", divisionSign(false), str);
+	gsub("-", SIGN_MINUS, str);
+	return str;
+}
+
+std::string QalculateQtSettings::unlocalizeExpression(std::string str) {
+	ParseOptions pa = evalops.parse_options; pa.base = 10;
+	str = CALCULATOR->unlocalizeExpression(str, pa);
+	CALCULATOR->parseSigns(str);
+	return str;
+}
+
 void QalculateQtSettings::updateMessagePrintOptions() {
 	PrintOptions message_printoptions = printops;
+	message_printoptions.is_approximate = NULL;
 	message_printoptions.interval_display = INTERVAL_DISPLAY_PLUSMINUS;
 	message_printoptions.show_ending_zeroes = false;
 	message_printoptions.base = 10;
@@ -1009,7 +1131,7 @@ void QalculateQtSettings::updateMessagePrintOptions() {
 	CALCULATOR->setMessagePrintOptions(message_printoptions);
 }
 
-MathLineEdit::MathLineEdit(QWidget *parent) : QLineEdit(parent) {
+MathLineEdit::MathLineEdit(QWidget *parent, bool unit_expression, bool function_expression) : QLineEdit(parent), b_unit(unit_expression), b_function(function_expression) {
 #ifndef _WIN32
 	setAttribute(Qt::WA_InputMethodEnabled, settings->enable_input_method);
 #endif
@@ -1019,7 +1141,7 @@ void MathLineEdit::keyPressEvent(QKeyEvent *event) {
 	if(event->modifiers() == Qt::NoModifier || event->modifiers() == Qt::GroupSwitchModifier || event->modifiers() == Qt::ShiftModifier || event->modifiers() == Qt::KeypadModifier) {
 		switch(event->key()) {
 			case Qt::Key_Asterisk: {
-				insert(SIGN_MULTIPLICATION);
+				insert(settings->multiplicationSign(b_unit));
 				return;
 			}
 			case Qt::Key_Minus: {
@@ -1041,6 +1163,14 @@ void MathLineEdit::keyPressEvent(QKeyEvent *event) {
 				}
 				break;
 			}
+			case Qt::Key_BraceLeft: {
+				if(!b_function) {return;}
+				break;
+			}
+			case Qt::Key_BraceRight: {
+				if(!b_function) {return;}
+				break;
+			}
 		}
 	}
 	if(event->key() == Qt::Key_Asterisk && (event->modifiers() == Qt::ControlModifier || event->modifiers() == (Qt::ControlModifier | Qt::KeypadModifier) || event->modifiers() == (Qt::ControlModifier | Qt::ShiftModifier))) {
