@@ -37,6 +37,7 @@ bool item_in_calculator(ExpressionItem *item);
 bool name_matches(ExpressionItem *item, const std::string &str);
 bool country_matches(Unit *u, const std::string &str, size_t minlength = 0);
 bool title_matches(ExpressionItem *item, const std::string &str, size_t minlength = 0);
+bool contains_plot_or_save(const std::string &str);
 
 enum {
 	TITLE_APP,
@@ -67,14 +68,17 @@ class QalculateQtSettings : QObject {
 		void updateStyle();
 		void updatePalette();
 		void updateMessagePrintOptions();
+		void updateFavourites();
 		bool checkExchangeRates(QWidget *parent);
 		void fetchExchangeRates(int timeout, int n = -1, QWidget *parent = NULL);
 		bool displayMessages(QWidget *parent);
 		bool isAnswerVariable(Variable *v);
 		void checkVersion(bool force, QWidget *parent);
 		void autoUpdate(std::string new_version, QWidget *parent);
-		const char *multiplicationSign();
+		const char *multiplicationSign(bool units = false);
 		const char *divisionSign(bool output = true);
+		std::string localizeExpression(std::string, bool unit_expression = false);
+		std::string unlocalizeExpression(std::string);
 
 		EvaluationOptions evalops;
 		PrintOptions printops;
@@ -95,6 +99,7 @@ class QalculateQtSettings : QObject {
 		bool keep_function_dialog_open;
 		bool save_defs_on_exit, save_mode_on_exit, clear_history_on_exit;
 		bool rpn_shown;
+		bool auto_calculate;
 		std::string custom_result_font, custom_expression_font, custom_keypad_font, custom_app_font;
 		KnownVariable *vans[5], *v_memory;
 		MathStructure *current_result;
@@ -105,6 +110,7 @@ class QalculateQtSettings : QObject {
 		QByteArray functions_geometry, functions_vsplitter_state, functions_hsplitter_state;
 		QByteArray units_geometry, units_vsplitter_state, units_hsplitter_state;
 		QByteArray variables_geometry, variables_vsplitter_state, variables_hsplitter_state;
+		QByteArray datasets_geometry, datasets_vsplitter_state, datasets_hsplitter_state;
 
 		PlotLegendPlacement default_plot_legend_placement;
 		PlotStyle default_plot_style;
@@ -116,6 +122,7 @@ class QalculateQtSettings : QObject {
 		QalculateDateTime last_version_check_date;
 		bool check_version;
 		std::string last_found_version;
+		int preferences_version[3];
 
 		std::vector<std::string> v_expression;
 		std::vector<bool> v_protected;
@@ -123,6 +130,13 @@ class QalculateQtSettings : QObject {
 		std::vector<std::vector<std::string> > v_result;
 		std::vector<std::vector<bool> > v_delresult;
 		std::vector<std::vector<int> > v_exact;
+		std::vector<MathFunction*> favourite_functions;
+		std::vector<Variable*> favourite_variables;
+		std::vector<Unit*> favourite_units;
+		std::vector<std::string> favourite_functions_pre;
+		std::vector<std::string> favourite_variables_pre;
+		std::vector<std::string> favourite_units_pre;
+		bool favourite_functions_changed, favourite_variables_changed, favourite_units_changed;
 
 };
 
@@ -130,11 +144,12 @@ class MathLineEdit : public QLineEdit {
 
 	public:
 
-		MathLineEdit(QWidget *parent = NULL);
+		MathLineEdit(QWidget *parent = NULL, bool unit_expression = false, bool function_expression = false);
 		virtual ~MathLineEdit();
 
 	protected:
 
+		bool b_unit, b_function;
 		void keyPressEvent(QKeyEvent*) override;
 
 };
