@@ -34,12 +34,16 @@ class QToolButton;
 class QTimer;
 class PreferencesDialog;
 class FunctionsDialog;
+class DataSetsDialog;
 class VariablesDialog;
 class UnitsDialog;
 class FPConversionDialog;
+class PercentageCalculationDialog;
 class PlotDialog;
+class PeriodicTableDialog;
 class CalendarConversionDialog;
 class QTableWidget;
+class QMenu;
 struct FunctionDialog;
 
 class QalculateWindow : public QMainWindow {
@@ -70,11 +74,14 @@ class QalculateWindow : public QMainWindow {
 		QSplitter *ehSplitter;
 		QLabel *statusLabel, *statusIconLabel;
 		PreferencesDialog *preferencesDialog;
+		DataSetsDialog *datasetsDialog;
 		FunctionsDialog *functionsDialog;
 		VariablesDialog *variablesDialog;
 		UnitsDialog *unitsDialog;
 		FPConversionDialog *fpConversionDialog;
+		PercentageCalculationDialog *percentageDialog;
 		PlotDialog *plotDialog;
+		PeriodicTableDialog *periodicTableDialog;
 		CalendarConversionDialog *calendarConversionDialog;
 
 		KeypadWidget *keypad;
@@ -83,7 +90,8 @@ class QalculateWindow : public QMainWindow {
 		QLabel *binLabel, *octLabel, *decLabel, *hexLabel;
 		QToolBar *tb;
 		QToolButton *menuAction, *modeAction;
-		QAction *toAction, *storeAction, *functionsAction, *keypadAction, *basesAction, *customOutputBaseAction, *customInputBaseAction;
+		QAction *toAction, *storeAction, *variablesAction, *functionsAction, *unitsAction, *plotAction, *fpAction, *calendarsAction, *percentageAction, *keypadAction, *basesAction, *customOutputBaseAction, *customInputBaseAction;
+		QMenu *variablesMenu, *functionsMenu, *unitsMenu;
 		QAction *assumptionTypeActions[5], *assumptionSignActions[6];
 		QSpinBox *customOutputBaseEdit, *customInputBaseEdit;
 		QTimer *ecTimer, *rfTimer;
@@ -199,6 +207,14 @@ class QalculateWindow : public QMainWindow {
 		void clearStack();
 		void registerChanged(int);
 		void calculateRPN(int);
+		void approximateResult();
+		void onExpressionStatusModeChanged();
+		void functionActivated();
+		void unitActivated();
+		void variableActivated();
+		void updateFunctionsMenu();
+		void updateUnitsMenu();
+		void updateVariablesMenu();
 
 	public slots:
 
@@ -217,6 +233,7 @@ class QalculateWindow : public QMainWindow {
 		void expressionFormatUpdated(bool);
 		void insertFunction(MathFunction*, QWidget* = NULL);
 		void newVariable();
+		void newUnit();
 		void newMatrix();
 		void newUnknown();
 		void newFunction();
@@ -224,12 +241,18 @@ class QalculateWindow : public QMainWindow {
 		void importCSV();
 		void exportCSV();
 		void editPreferences();
+		void onDatasetsChanged();
+		void onFunctionsChanged();
+		void insertProperty(DataObject*, DataProperty*);
+		void openDatasets();
 		void openFunctions();
 		void openUnits();
 		void openVariables();
 		void applyFunction(MathFunction*);
 		void openFPConversion();
+		void openPercentageCalculation();
 		void openPlot();
+		void openPeriodicTable();
 		void negate();
 		void checkVersion();
 		void reportBug();
