@@ -23,15 +23,16 @@
 #include <QKeyEvent>
 #include <QLabel>
 #include <QComboBox>
+#include <QCheckBox>
 #include <QApplication>
 #include <QDebug>
 
 #include "qalculateqtsettings.h"
 #include "unitsdialog.h"
 #include "itemproxymodel.h"
-//#include "uniteditdialog.h"
+#include "uniteditdialog.h"
 
-UnitsDialog::UnitsDialog(QWidget *parent) : QDialog(parent) {
+UnitsDialog::UnitsDialog(QWidget *parent) : QDialog(parent, Qt::Window) {
 	last_from = true;
 	QVBoxLayout *topbox = new QVBoxLayout(this);
 	setWindowTitle(tr("Units"));
@@ -67,7 +68,9 @@ UnitsDialog::UnitsDialog(QWidget *parent) : QDialog(parent) {
 	searchEdit = new QLineEdit(this);
 	searchEdit->addAction(LOAD_ICON("edit-find"), QLineEdit::LeadingPosition);
 #ifdef _WIN32
+#	if (QT_VERSION < QT_VERSION_CHECK(6, 2, 0))
 	searchEdit->setTextMargins(22, 0, 0, 0);
+#	endif
 #endif
 	searchEdit->installEventFilter(this);
 	vbox->addWidget(searchEdit, 0);
@@ -89,9 +92,8 @@ UnitsDialog::UnitsDialog(QWidget *parent) : QDialog(parent) {
 	convertButton = new QPushButton(tr("Convert"), this); box->addWidget(convertButton); connect(convertButton, SIGNAL(clicked()), this, SLOT(convertClicked()));
 	insertButton = new QPushButton(tr("Insert"), this); box->addWidget(insertButton); connect(insertButton, SIGNAL(clicked()), this, SLOT(insertClicked()));
 	insertButton->setDefault(true);
-	newButton->hide();
-	editButton->hide();
-	delButton->hide();
+	box->addSpacing(24);
+	favouriteButton = new QCheckBox(tr("Favorite"), this); box->addWidget(favouriteButton); connect(favouriteButton, SIGNAL(clicked()), this, SLOT(favouriteClicked()));
 	box->addStretch(1);
 	hbox->addLayout(box, 0);
 	QGridLayout *grid = new QGridLayout();
@@ -125,8 +127,6 @@ UnitsDialog::UnitsDialog(QWidget *parent) : QDialog(parent) {
 	selected_category = "All";
 	updateUnits();
 	unitsView->setFocus();
-	unitsModel->setFilter("All");
-	toModel->setFilter("All");
 	connect(searchEdit, SIGNAL(textChanged(const QString&)), this, SLOT(searchChanged(const QString&)));
 	connect(fromEdit, SIGNAL(textEdited(const QString&)), this, SLOT(fromChanged()));
 	connect(toEdit, SIGNAL(textEdited(const QString&)), this, SLOT(toChanged()));
@@ -135,7 +135,6 @@ UnitsDialog::UnitsDialog(QWidget *parent) : QDialog(parent) {
 	connect(categoriesView, SIGNAL(currentItemChanged(QTreeWidgetItem*, QTreeWidgetItem*)), this, SLOT(selectedCategoryChanged(QTreeWidgetItem*, QTreeWidgetItem*)));
 	connect(unitsView->selectionModel(), SIGNAL(currentChanged(const QModelIndex&, const QModelIndex&)), this, SLOT(selectedUnitChanged(const QModelIndex&, const QModelIndex&)));
 	connect(unitsView, SIGNAL(doubleClicked(const QModelIndex&)), this, SLOT(onUnitActivated(const QModelIndex&)));
-	selectedUnitChanged(QModelIndex(), QModelIndex());
 	if(!settings->units_geometry.isEmpty()) restoreGeometry(settings->units_geometry);
 	else resize(900, 800);
 	if(!settings->units_vsplitter_state.isEmpty()) vsplitter->restoreState(settings->units_vsplitter_state);
@@ -282,22 +281,196 @@ void UnitsDialog::searchChanged(const QString &str) {
 	unitsView->selectionModel()->setCurrentIndex(unitsModel->index(0, 0), QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
 	if(str.isEmpty()) unitsView->setFocus();
 }
-void UnitsDialog::newClicked() {}
+#define SET_TO_STR 	if(u->isCurrency()) qstr = QString::fromStdString(u->referenceName());\
+			else qstr = QString::fromStdString(u->print(true, true, settings->printops.use_unicode_signs, &can_display_unicode_string_function, (void*) toCombo));\
+			if(u->subtype() == SUBTYPE_COMPOSITE_UNIT) {\
+				qstr.replace("^-1", "⁻" SIGN_POWER_1);\
+				qstr.replace("^-2", "⁻" SIGN_POWER_2);\
+				qstr.replace("^-3", "⁻" SIGN_POWER_3);\
+				qstr.replace("^-4", "⁻" SIGN_POWER_4);\
+				qstr.replace("^4", SIGN_POWER_4);\
+				qstr.remove("_unit");\
+			}
+
+void UnitsDialog::newClicked() {
+	ExpressionItem *replaced_item = NULL;
+	Unit *u = UnitEditDialog::newUnit(this, &replaced_item);
+	if(u) {
+		if(replaced_item) {
+			if(!CALCULATOR->stillHasVariable((Variable*) replaced_item)) {
+				emit variableRemoved((Variable*) replaced_item);
+			} else if(replaced_item == u || !CALCULATOR->stillHasUnit((Unit*) replaced_item) || (replaced_item->type() == TYPE_UNIT && !CALCULATOR->hasUnit((Unit*) replaced_item))) {
+				QModelIndexList list = sourceModel->match(sourceModel->index(0, 0), Qt::UserRole, QVariant::fromValue((void*) replaced_item), 1, Qt::MatchExactly);
+				if(!list.isEmpty()) sourceModel->removeRow(list[0].row());
+				list = toSourceModel->match(toSourceModel->index(0, 0), Qt::UserRole, QVariant::fromValue((void*) replaced_item), 1, Qt::MatchExactly);
+				if(!list.isEmpty()) toSourceModel->removeRow(list[0].row());
+			} else if(replaced_item->type() == TYPE_VARIABLE) {
+				if(!CALCULATOR->hasVariable((Variable*) replaced_item)) emit variableRemoved((Variable*) replaced_item);
+				else if(!replaced_item->isActive()) emit variableDeactivated((Variable*) replaced_item);
+			} else if(!replaced_item->isActive()) {
+				QList<QTreeWidgetItem*> list = categoriesView->findItems("Inactive", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+				if(list.isEmpty()) {QStringList l; l << tr("Inactive"); l << "Inactive"; new QTreeWidgetItem(categoriesView, l);}
+			}
+		}
+		selected_item = u;
+		if(u->category().empty()) {
+			QList<QTreeWidgetItem*> list = categoriesView->findItems("Uncategorized", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+			if(list.isEmpty()) {QStringList l; l << tr("Uncategorized"); l << "Uncategorized"; new QTreeWidgetItem(categoriesView->topLevelItem(2), l);}
+		} else if(u->category() != CALCULATOR->temporaryCategory()) {
+			QList<QTreeWidgetItem*> list = categoriesView->findItems(QString("/") + QString::fromStdString(u->category()), Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+			if(list.isEmpty()) {
+				if(selected_category != "All") selected_category = "User items";
+				updateUnits();
+				emit itemsChanged();
+				return;
+			}
+		}
+		QString qstr;
+		SET_TO_STR
+		QStandardItem *item = new QStandardItem(qstr);
+		item->setEditable(false);
+		item->setData(QVariant::fromValue((void*) u), Qt::UserRole);
+		toSourceModel->appendRow(item);
+		item = new QStandardItem(QString::fromStdString(u->title(true, settings->printops.use_unicode_signs, &can_display_unicode_string_function, (void*) unitsView)));
+		item->setEditable(false);
+		item->setData(QVariant::fromValue((void*) u), Qt::UserRole);
+		sourceModel->appendRow(item);
+		if(selected_category != "All" && selected_category != "User items" && selected_category != std::string("/") + u->category() && (selected_category != "Uncategorized" || !u->category().empty())) {
+			QList<QTreeWidgetItem*> list = categoriesView->findItems("User items", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+			if(!list.isEmpty()) {
+				categoriesView->setCurrentItem(list[0], 0, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
+			}
+		} else {
+			unitsModel->invalidate();
+			toModel->invalidate();
+		}
+		sourceModel->sort(0);
+		QModelIndex index = unitsModel->mapFromSource(item->index());
+		if(index.isValid()) {
+			unitsView->selectionModel()->setCurrentIndex(index, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
+			unitsView->scrollTo(index);
+		}
+		if(u != replaced_item && !u->isHidden()) settings->favourite_units.push_back(u);
+		emit itemsChanged();
+	}
+}
 void UnitsDialog::unitRemoved(Unit *u) {
 	QModelIndexList list = sourceModel->match(sourceModel->index(0, 0), Qt::UserRole, QVariant::fromValue((void*) u), 1, Qt::MatchExactly);
 	if(!list.isEmpty()) sourceModel->removeRow(list[0].row());
+	list = toSourceModel->match(toSourceModel->index(0, 0), Qt::UserRole, QVariant::fromValue((void*) u), 1, Qt::MatchExactly);
+	if(!list.isEmpty()) toSourceModel->removeRow(list[0].row());
 }
 void UnitsDialog::unitDeactivated(Unit*) {
-	ADD_INACTIVE_CATEGORY
+	QList<QTreeWidgetItem*> list = categoriesView->findItems("Inactive", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+	if(list.isEmpty()) {QStringList l; l << tr("Inactive"); l << "Inactive"; new QTreeWidgetItem(categoriesView, l);}
 	unitsModel->invalidate();
+	toModel->invalidate();
+}
+void UnitsDialog::editClicked() {
+	QModelIndex index = unitsView->selectionModel()->currentIndex();
+	if(!index.isValid()) return;
+	Unit *old_u = (Unit*) index.data(Qt::UserRole).value<void*>();
+	if(!old_u) return;
+	ExpressionItem *replaced_item = NULL;
+	Unit *u = UnitEditDialog::editUnit(this, old_u, &replaced_item);
+	if(u) {
+		sourceModel->removeRow(unitsModel->mapToSource(unitsView->selectionModel()->currentIndex()).row());
+		QModelIndexList list = toSourceModel->match(toSourceModel->index(0, 0), Qt::UserRole, QVariant::fromValue((void*) old_u), 1, Qt::MatchExactly);
+		if(!list.isEmpty()) toSourceModel->removeRow(list[0].row());
+		if(replaced_item) {
+			if(!CALCULATOR->stillHasVariable((Variable*) replaced_item)) {
+				emit variableRemoved((Variable*) replaced_item);
+			} else if(!CALCULATOR->stillHasUnit((Unit*) replaced_item) || (replaced_item->type() == TYPE_UNIT && !CALCULATOR->hasUnit((Unit*) replaced_item))) {
+				list = sourceModel->match(sourceModel->index(0, 0), Qt::UserRole, QVariant::fromValue((void*) replaced_item), 1, Qt::MatchExactly);
+				if(!list.isEmpty()) sourceModel->removeRow(list[0].row());
+				list = toSourceModel->match(toSourceModel->index(0, 0), Qt::UserRole, QVariant::fromValue((void*) replaced_item), 1, Qt::MatchExactly);
+				if(!list.isEmpty()) toSourceModel->removeRow(list[0].row());
+			} else if(replaced_item->type() == TYPE_VARIABLE) {
+				if(!CALCULATOR->hasVariable((Variable*) replaced_item)) emit variableRemoved((Variable*) replaced_item);
+				else if(!replaced_item->isActive()) emit variableDeactivated((Variable*) replaced_item);
+			} else if(!replaced_item->isActive()) {
+				QList<QTreeWidgetItem*> list = categoriesView->findItems("Inactive", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+				if(list.isEmpty()) {QStringList l; l << tr("Inactive"); l << "Inactive"; new QTreeWidgetItem(categoriesView, l);}
+			}
+		}
+		selected_item = u;
+		if(u->category().empty()) {
+			QList<QTreeWidgetItem*> list = categoriesView->findItems("Uncategorized", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+			if(list.isEmpty()) {QStringList l; l << tr("Uncategorized"); l << "Uncategorized"; new QTreeWidgetItem(categoriesView->topLevelItem(2), l);}
+		} else if(u->category() != CALCULATOR->temporaryCategory()) {
+			QList<QTreeWidgetItem*> list = categoriesView->findItems(QString("/") + QString::fromStdString(u->category()), Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+			if(list.isEmpty()) {
+				if(selected_category != "All") selected_category = "User items";
+				updateUnits();
+				emit itemsChanged();
+				return;
+			}
+		}
+		QString qstr;
+		SET_TO_STR
+		QStandardItem *item = new QStandardItem(qstr);
+		item->setEditable(false);
+		item->setData(QVariant::fromValue((void*) u), Qt::UserRole);
+		toSourceModel->appendRow(item);
+		item = new QStandardItem(QString::fromStdString(u->title(true, settings->printops.use_unicode_signs, &can_display_unicode_string_function, (void*) unitsView)));
+		item->setEditable(false);
+		item->setData(QVariant::fromValue((void*) u), Qt::UserRole);
+		sourceModel->appendRow(item);
+		if(selected_category != "All" && selected_category != "User items" && selected_category != std::string("/") + u->category() && (selected_category != "Uncategorized" || !u->category().empty())) {
+			QList<QTreeWidgetItem*> list = categoriesView->findItems("User items", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+			if(!list.isEmpty()) {
+				categoriesView->setCurrentItem(list[0], 0, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
+			}
+		} else {
+			unitsModel->invalidate();
+			toModel->invalidate();
+		}
+		sourceModel->sort(0);
+		toSourceModel->sort(0);
+		QModelIndex index = unitsModel->mapFromSource(item->index());
+		if(index.isValid()) {
+			unitsView->selectionModel()->setCurrentIndex(index, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
+			unitsView->scrollTo(index);
+		}
+		emit itemsChanged();
+	}
+}
+void UnitsDialog::favouriteClicked() {
+	QModelIndex index = unitsView->selectionModel()->currentIndex();
+	if(!index.isValid()) return;
+	Unit *u = (Unit*) index.data(Qt::UserRole).value<void*>();
+	if(!u) return;
+	if(favouriteButton->isChecked()) {
+		settings->favourite_units.push_back(u);
+	} else {
+		for(size_t i = 0; i < settings->favourite_units.size(); i++) {
+			if(settings->favourite_units[i] == u) {
+				settings->favourite_units.erase(settings->favourite_units.begin() + i);
+				break;
+			}
+		}
+		if(selected_category == "Favorites") {
+			if(settings->favourite_units.empty()) {
+				QList<QTreeWidgetItem*> list = categoriesView->findItems("All", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+				if(!list.isEmpty()) categoriesView->setCurrentItem(list[0], 0, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
+			} else {
+				selected_item = NULL;
+				unitsModel->invalidate();
+				selectedUnitChanged(unitsView->selectionModel()->currentIndex(), unitsView->selectionModel()->currentIndex());
+			}
+		}
+	}
+	settings->favourite_units_changed = true;
+	emit itemsChanged();
 }
-void UnitsDialog::editClicked() {}
 void UnitsDialog::delClicked() {
 	QModelIndex index = unitsView->selectionModel()->currentIndex();
 	if(!index.isValid()) return;
 	Unit *u = (Unit*) index.data(Qt::UserRole).value<void*>();
 	if(u && u->isLocal()) {
 		sourceModel->removeRow(unitsModel->mapToSource(unitsView->selectionModel()->currentIndex()).row());
+		QModelIndexList list = toSourceModel->match(toSourceModel->index(0, 0), Qt::UserRole, QVariant::fromValue((void*) u), 1, Qt::MatchExactly);
+		if(!list.isEmpty()) toSourceModel->removeRow(list[0].row());
 		selected_item = NULL;
 		u->destroy();
 		emit itemsChanged();
@@ -427,10 +600,18 @@ void UnitsDialog::selectedUnitChanged(const QModelIndex &index, const QModelInde
 					deactivateButton->setText(tr("Activate"));
 				}
 			}
-			//editButton->setEnabled(!u->isBuiltin());
+			editButton->setEnabled(true);
 			insertButton->setEnabled(u->isActive());
 			convertButton->setEnabled(u->isActive());
 			delButton->setEnabled(u->isLocal());
+			favouriteButton->setEnabled(true);
+			favouriteButton->setChecked(false);
+			for(size_t i = 0; i < settings->favourite_units.size(); i++) {
+				if(settings->favourite_units[i] == u) {
+					favouriteButton->setChecked(true);
+					break;
+				}
+			}
 			deactivateButton->setEnabled(true);
 			descriptionView->setHtml(QString::fromStdString(str));
 			std::string to_filter;
@@ -460,16 +641,7 @@ void UnitsDialog::selectedUnitChanged(const QModelIndex &index, const QModelInde
 			toEdit->setEnabled(u->isActive());
 			toCombo->setEnabled(u->isActive());
 			QString qstr;
-			if(u->isCurrency()) qstr = QString::fromStdString(u->referenceName());
-			else qstr = QString::fromStdString(u->print(true, true, settings->printops.use_unicode_signs, &can_display_unicode_string_function, (void*) fromLabel));
-			if(u->subtype() == SUBTYPE_COMPOSITE_UNIT) {
-				qstr.replace("^-1", "⁻" SIGN_POWER_1);
-				qstr.replace("^-2", "⁻" SIGN_POWER_2);
-				qstr.replace("^-3", "⁻" SIGN_POWER_3);
-				qstr.replace("^-4", "⁻" SIGN_POWER_4);
-				qstr.replace("^4", SIGN_POWER_4);
-				qstr.remove("_unit");
-			}
+			SET_TO_STR
 			fromLabel->setText(qstr);
 			fromUnitChanged();
 			return;
@@ -484,6 +656,8 @@ void UnitsDialog::selectedUnitChanged(const QModelIndex &index, const QModelInde
 	insertButton->setEnabled(false);
 	convertButton->setEnabled(false);
 	delButton->setEnabled(false);
+	favouriteButton->setEnabled(true);
+	favouriteButton->setChecked(false);
 	deactivateButton->setEnabled(false);
 	descriptionView->clear();
 	selected_item = NULL;
@@ -531,7 +705,9 @@ void UnitsDialog::updateUnits() {
 	unit_cats.parent = NULL;
 	bool has_inactive = false, has_uncat = false;
 	std::list<tree_struct>::iterator it;
+	QStandardItem *item_sel = NULL;
 
+	unitsView->selectionModel()->blockSignals(true);
 	sourceModel->clear();
 	sourceModel->setColumnCount(1);
 	sourceModel->setHorizontalHeaderItem(0, new QStandardItem(tr("Unit")));
@@ -578,39 +754,43 @@ void UnitsDialog::updateUnits() {
 					cat_i_prev = cat_i + 1;
 					cat_i = cat.find("/", cat_i_prev);
 				}
-			} else if(!u->isLocal()) {
+			} else {
 				has_uncat = true;
 			}
 		}
-		QStandardItem *item = new QStandardItem(QString::fromStdString(u->title(true)));
+		QStandardItem *item = new QStandardItem(QString::fromStdString(u->title(true, settings->printops.use_unicode_signs, &can_display_unicode_string_function, (void*) unitsView)));
 		item->setEditable(false);
 		item->setData(QVariant::fromValue((void*) u), Qt::UserRole);
 		sourceModel->appendRow(item);
-		if(u == selected_item) unitsView->selectionModel()->setCurrentIndex(unitsModel->mapFromSource(item->index()), QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
-		
-		if(u->isCurrency()) qstr = QString::fromStdString(u->referenceName());
-		else qstr = QString::fromStdString(u->print(true, true, settings->printops.use_unicode_signs, &can_display_unicode_string_function, (void*) toCombo));
-		if(u->subtype() == SUBTYPE_COMPOSITE_UNIT) {
-			qstr.replace("^-1", "⁻" SIGN_POWER_1);
-			qstr.replace("^-2", "⁻" SIGN_POWER_2);
-			qstr.replace("^-3", "⁻" SIGN_POWER_3);
-			qstr.replace("^-4", "⁻" SIGN_POWER_4);
-			qstr.replace("^4", SIGN_POWER_4);
-			qstr.remove("_unit");
-		}
+		if(u == selected_item) item_sel = item;
+
+		SET_TO_STR
 		item = new QStandardItem(qstr);
 		item->setEditable(false);
 		item->setData(QVariant::fromValue((void*) u), Qt::UserRole);
 		toSourceModel->appendRow(item);
 	}
+	unitsView->selectionModel()->blockSignals(false);
 	sourceModel->sort(0);
 	toSourceModel->sort(0);
 	unit_cats.sort();
 
+	categoriesView->blockSignals(true);
 	categoriesView->clear();
+	categoriesView->blockSignals(false);
 	QTreeWidgetItem *iter, *iter2, *iter3;
 	QStringList l;
-	l << tr("All", "All units"); l << "All";
+	l.clear(); l << tr("Favorites"); l << "Favorites";
+	iter = new QTreeWidgetItem(categoriesView, l);
+	if(selected_category == "Favorites") {
+		iter->setSelected(true);
+	}
+	l.clear(); l << tr("User units"); l << "User items";
+	iter = new QTreeWidgetItem(categoriesView, l);
+	if(selected_category == "User items") {
+		iter->setSelected(true);
+	}
+	l.clear(); l << tr("All", "All units"); l << "All";
 	iter3 = new QTreeWidgetItem(categoriesView, l);
 	tree_struct *item, *item2;
 	unit_cats.it = unit_cats.items.begin();
@@ -659,11 +839,6 @@ void UnitsDialog::updateUnits() {
 			iter->setSelected(true);
 		}
 	}
-	l.clear(); l << tr("User units"); l << "User items";
-	iter = new QTreeWidgetItem(iter3, l);
-	if(selected_category == "User items") {
-		iter->setSelected(true);
-	}
 	if(has_inactive) {
 		//add "Inactive" category if there are inactive units
 		l.clear(); l << tr("Inactive"); l << "Inactive";
@@ -672,12 +847,24 @@ void UnitsDialog::updateUnits() {
 			iter->setSelected(true);
 		}
 	}
+	iter3->setExpanded(true);
 	if(categoriesView->selectedItems().isEmpty()) {
 		//if no category has been selected (previously selected has been renamed/deleted), select "All"
 		selected_category = "All";
-		iter3->setExpanded(true);
 		iter3->setSelected(true);
 	}
+	searchEdit->blockSignals(true);
+	searchEdit->clear();
+	searchEdit->blockSignals(false);
+	unitsModel->setFilter(selected_category);
+	QModelIndex index;
+	if(item_sel) index = unitsModel->mapFromSource(item_sel->index());
+	if(!index.isValid()) index = unitsModel->index(0, 0);
+	if(index.isValid()) {
+		unitsView->selectionModel()->setCurrentIndex(index, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
+		unitsView->scrollTo(index);
+	}
+	selectedUnitChanged(index, QModelIndex());
 }
 void UnitsDialog::setSearch(const QString &str) {
 	searchEdit->setText(str);
@@ -689,4 +876,3 @@ void UnitsDialog::selectCategory(std::string str) {
 	}
 }
 
-
