@@ -19,6 +19,7 @@
 #include <QTreeWidget>
 #include <QTreeWidgetItem>
 #include <QPushButton>
+#include <QCheckBox>
 #include <QLineEdit>
 #include <QKeyEvent>
 #include <QMenu>
@@ -32,7 +33,7 @@
 #include "variableeditdialog.h"
 #include "csvdialog.h"
 
-VariablesDialog::VariablesDialog(QWidget *parent) : QDialog(parent) {
+VariablesDialog::VariablesDialog(QWidget *parent) : QDialog(parent, Qt::Window) {
 	QVBoxLayout *topbox = new QVBoxLayout(this);
 	setWindowTitle(tr("Variables"));
 	QHBoxLayout *hbox = new QHBoxLayout();
@@ -67,7 +68,9 @@ VariablesDialog::VariablesDialog(QWidget *parent) : QDialog(parent) {
 	searchEdit = new QLineEdit(this);
 	searchEdit->addAction(LOAD_ICON("edit-find"), QLineEdit::LeadingPosition);
 #ifdef _WIN32
+#	if (QT_VERSION < QT_VERSION_CHECK(6, 2, 0))
 	searchEdit->setTextMargins(22, 0, 0, 0);
+#	endif
 #endif
 	searchEdit->installEventFilter(this);
 	vbox->addWidget(searchEdit, 0);
@@ -94,6 +97,8 @@ VariablesDialog::VariablesDialog(QWidget *parent) : QDialog(parent) {
 	box->addSpacing(24);
 	insertButton = new QPushButton(tr("Insert"), this); box->addWidget(insertButton); connect(insertButton, SIGNAL(clicked()), this, SLOT(insertClicked()));
 	insertButton->setDefault(true);
+	box->addSpacing(24);
+	favouriteButton = new QCheckBox(tr("Favorite"), this); box->addWidget(favouriteButton); connect(favouriteButton, SIGNAL(clicked()), this, SLOT(favouriteClicked()));
 	box->addStretch(1);
 	hbox->addLayout(box, 0);
 	QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Close, Qt::Horizontal, this);
@@ -101,13 +106,11 @@ VariablesDialog::VariablesDialog(QWidget *parent) : QDialog(parent) {
 	selected_category = "All";
 	updateVariables();
 	variablesView->setFocus();
-	variablesModel->setFilter("All");
 	connect(searchEdit, SIGNAL(textChanged(const QString&)), this, SLOT(searchChanged(const QString&)));
 	connect(buttonBox->button(QDialogButtonBox::Close), SIGNAL(clicked()), this, SLOT(reject()));
 	connect(categoriesView, SIGNAL(currentItemChanged(QTreeWidgetItem*, QTreeWidgetItem*)), this, SLOT(selectedCategoryChanged(QTreeWidgetItem*, QTreeWidgetItem*)));
 	connect(variablesView->selectionModel(), SIGNAL(currentChanged(const QModelIndex&, const QModelIndex&)), this, SLOT(selectedVariableChanged(const QModelIndex&, const QModelIndex&)));
 	connect(variablesView, SIGNAL(doubleClicked(const QModelIndex&)), this, SLOT(insertClicked()));
-	selectedVariableChanged(QModelIndex(), QModelIndex());
 	if(!settings->variables_geometry.isEmpty()) restoreGeometry(settings->variables_geometry);
 	else resize(900, 700);
 	if(!settings->variables_vsplitter_state.isEmpty()) vsplitter->restoreState(settings->variables_vsplitter_state);
@@ -190,15 +193,28 @@ void VariablesDialog::newVariable(int type) {
 				if(!CALCULATOR->hasUnit((Unit*) replaced_item)) emit unitRemoved((Unit*) replaced_item);
 				else if(!replaced_item->isActive()) emit unitDeactivated((Unit*) replaced_item);
 			} else if(!replaced_item->isActive()) {
-				ADD_INACTIVE_CATEGORY
+				QList<QTreeWidgetItem*> list = categoriesView->findItems("Inactive", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+				if(list.isEmpty()) {QStringList l; l << tr("Inactive"); l << "Inactive"; new QTreeWidgetItem(categoriesView, l);}
 			}
 		}
 		selected_item = v;
-		QStandardItem *item = new QStandardItem(QString::fromStdString(v->title(true)));
+		if(v->category().empty()) {
+			QList<QTreeWidgetItem*> list = categoriesView->findItems("Uncategorized", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+			if(list.isEmpty()) {QStringList l; l << tr("Uncategorized"); l << "Uncategorized"; new QTreeWidgetItem(categoriesView->topLevelItem(2), l);}
+		} else if(v->category() != CALCULATOR->temporaryCategory()) {
+			QList<QTreeWidgetItem*> list = categoriesView->findItems(QString("/") + QString::fromStdString(v->category()), Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+			if(list.isEmpty()) {
+				if(selected_category != "All") selected_category = "User items";
+				updateVariables();
+				emit itemsChanged();
+				return;
+			}
+		}
+		QStandardItem *item = new QStandardItem(QString::fromStdString(v->title(true, settings->printops.use_unicode_signs, &can_display_unicode_string_function, (void*) variablesView)));
 		item->setEditable(false);
 		item->setData(QVariant::fromValue((void*) v), Qt::UserRole);
 		sourceModel->appendRow(item);
-		if(selected_category != "All" && selected_category != "User items" && selected_category != std::string("/") + v->category()) {
+		if(selected_category != "All" && selected_category != "User items" && selected_category != std::string("/") + v->category() && (selected_category != "Uncategorized" || !v->category().empty())) {
 			QList<QTreeWidgetItem*> list = categoriesView->findItems("User items", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
 			if(!list.isEmpty()) {
 				categoriesView->setCurrentItem(list[0], 0, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
@@ -212,6 +228,7 @@ void VariablesDialog::newVariable(int type) {
 			variablesView->selectionModel()->setCurrentIndex(index, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
 			variablesView->scrollTo(index);
 		}
+		if(v != replaced_item && !v->isHidden()) settings->favourite_variables.push_back(v);
 		emit itemsChanged();
 	}
 }
@@ -221,7 +238,8 @@ void VariablesDialog::variableRemoved(Variable *v) {
 	if(!list.isEmpty()) sourceModel->removeRow(list[0].row());
 }
 void VariablesDialog::variableDeactivated(Variable*) {
-	ADD_INACTIVE_CATEGORY
+	QList<QTreeWidgetItem*> list = categoriesView->findItems("Inactive", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+	if(list.isEmpty()) {QStringList l; l << tr("Inactive"); l << "Inactive"; new QTreeWidgetItem(categoriesView, l);}
 	variablesModel->invalidate();
 }
 void VariablesDialog::editClicked() {
@@ -248,15 +266,28 @@ void VariablesDialog::editClicked() {
 				if(!CALCULATOR->hasUnit((Unit*) replaced_item)) emit unitRemoved((Unit*) replaced_item);
 				else if(!replaced_item->isActive()) emit unitDeactivated((Unit*) replaced_item);
 			} else if(!replaced_item->isActive()) {
-				ADD_INACTIVE_CATEGORY
+				QList<QTreeWidgetItem*> list = categoriesView->findItems("Inactive", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+				if(list.isEmpty()) {QStringList l; l << tr("Inactive"); l << "Inactive"; new QTreeWidgetItem(categoriesView, l);}
 			}
 		}
-		QStandardItem *item = new QStandardItem(QString::fromStdString(v->title(true)));
+		selected_item = v;
+		if(v->category().empty()) {
+			QList<QTreeWidgetItem*> list = categoriesView->findItems("Uncategorized", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+			if(list.isEmpty()) {QStringList l; l << tr("Uncategorized"); l << "Uncategorized"; new QTreeWidgetItem(categoriesView->topLevelItem(2), l);}
+		} else if(v->category() != CALCULATOR->temporaryCategory()) {
+			QList<QTreeWidgetItem*> list = categoriesView->findItems(QString("/") + QString::fromStdString(v->category()), Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+			if(list.isEmpty()) {
+				if(selected_category != "All") selected_category = "User items";
+				updateVariables();
+				emit itemsChanged();
+				return;
+			}
+		}
+		QStandardItem *item = new QStandardItem(QString::fromStdString(v->title(true, settings->printops.use_unicode_signs, &can_display_unicode_string_function, (void*) variablesView)));
 		item->setEditable(false);
 		item->setData(QVariant::fromValue((void*) v), Qt::UserRole);
 		sourceModel->appendRow(item);
-		selected_item = v;
-		if(selected_category != "All" && selected_category != "User items" && selected_category != std::string("/") + v->category()) {
+		if(selected_category != "All" && selected_category != "User items" && selected_category != std::string("/") + v->category() && (selected_category != "Uncategorized" || !v->category().empty())) {
 			QList<QTreeWidgetItem*> list = categoriesView->findItems("User items", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
 			if(!list.isEmpty()) {
 				categoriesView->setCurrentItem(list[0], 0, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
@@ -273,6 +304,34 @@ void VariablesDialog::editClicked() {
 		emit itemsChanged();
 	}
 }
+void VariablesDialog::favouriteClicked() {
+	QModelIndex index = variablesView->selectionModel()->currentIndex();
+	if(!index.isValid()) return;
+	Variable *v = (Variable*) index.data(Qt::UserRole).value<void*>();
+	if(!v) return;
+	if(favouriteButton->isChecked()) {
+		settings->favourite_variables.push_back(v);
+	} else {
+		for(size_t i = 0; i < settings->favourite_variables.size(); i++) {
+			if(settings->favourite_variables[i] == v) {
+				settings->favourite_variables.erase(settings->favourite_variables.begin() + i);
+				break;
+			}
+		}
+		if(selected_category == "Favorites") {
+			if(settings->favourite_variables.empty()) {
+				QList<QTreeWidgetItem*> list = categoriesView->findItems("All", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+				if(!list.isEmpty()) categoriesView->setCurrentItem(list[0], 0, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
+			} else {
+				selected_item = NULL;
+				variablesModel->invalidate();
+				selectedVariableChanged(variablesView->selectionModel()->currentIndex(), variablesView->selectionModel()->currentIndex());
+			}
+		}
+	}
+	settings->favourite_variables_changed = true;
+	emit itemsChanged();
+}
 void VariablesDialog::delClicked() {
 	QModelIndex index = variablesView->selectionModel()->currentIndex();
 	if(!index.isValid()) return;
@@ -394,6 +453,14 @@ void VariablesDialog::selectedVariableChanged(const QModelIndex &index, const QM
 			editButton->setEnabled(!v->isBuiltin());
 			exportButton->setEnabled(v->isKnown() && ((KnownVariable*) v)->get().isVector());
 			insertButton->setEnabled(v->isActive());
+			favouriteButton->setEnabled(true);
+			favouriteButton->setChecked(false);
+			for(size_t i = 0; i < settings->favourite_variables.size(); i++) {
+				if(settings->favourite_variables[i] == v) {
+					favouriteButton->setChecked(true);
+					break;
+				}
+			}
 			delButton->setEnabled(v->isLocal());
 			deactivateButton->setEnabled(!settings->isAnswerVariable(v) && v != settings->v_memory);
 			descriptionView->setHtml(QString::fromStdString(str));
@@ -404,6 +471,8 @@ void VariablesDialog::selectedVariableChanged(const QModelIndex &index, const QM
 	insertButton->setEnabled(false);
 	exportButton->setEnabled(false);
 	delButton->setEnabled(false);
+	favouriteButton->setEnabled(true);
+	favouriteButton->setChecked(false);
 	deactivateButton->setEnabled(false);
 	descriptionView->clear();
 	selected_item = NULL;
@@ -446,7 +515,9 @@ void VariablesDialog::updateVariables() {
 	variable_cats.parent = NULL;
 	bool has_inactive = false, has_uncat = false;
 	std::list<tree_struct>::iterator it;
+	QStandardItem *item_sel = NULL;
 
+	variablesView->selectionModel()->blockSignals(true);
 	sourceModel->clear();
 	sourceModel->setColumnCount(1);
 	sourceModel->setHorizontalHeaderItem(0, new QStandardItem(tr("Variable")));
@@ -490,23 +561,36 @@ void VariablesDialog::updateVariables() {
 					cat_i_prev = cat_i + 1;
 					cat_i = cat.find("/", cat_i_prev);
 				}
-			} else if(!v->isLocal()) {
+			} else {
 				has_uncat = true;
 			}
 		}
-		QStandardItem *item = new QStandardItem(QString::fromStdString(v->title(true)));
+		QStandardItem *item = new QStandardItem(QString::fromStdString(v->title(true, settings->printops.use_unicode_signs, &can_display_unicode_string_function, (void*) variablesView)));
 		item->setEditable(false);
 		item->setData(QVariant::fromValue((void*) v), Qt::UserRole);
 		sourceModel->appendRow(item);
-		if(v == selected_item) variablesView->selectionModel()->setCurrentIndex(variablesModel->mapFromSource(item->index()), QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
+		if(v == selected_item) item_sel = item;
 	}
+	variablesView->selectionModel()->blockSignals(false);
 	sourceModel->sort(0);
 	variable_cats.sort();
 
+	categoriesView->blockSignals(true);
 	categoriesView->clear();
+	categoriesView->blockSignals(false);
 	QTreeWidgetItem *iter, *iter2, *iter3;
 	QStringList l;
-	l << tr("All", "All variables"); l << "All";
+	l.clear(); l << tr("Favorites"); l << "Favorites";
+	iter = new QTreeWidgetItem(categoriesView, l);
+	if(selected_category == "Favorites") {
+		iter->setSelected(true);
+	}
+	l.clear(); l << tr("User variables"); l << "User items";
+	iter = new QTreeWidgetItem(categoriesView, l);
+	if(selected_category == "User items") {
+		iter->setSelected(true);
+	}
+	l.clear(); l << tr("All", "All variables"); l << "All";
 	iter3 = new QTreeWidgetItem(categoriesView, l);
 	tree_struct *item, *item2;
 	variable_cats.it = variable_cats.items.begin();
@@ -555,11 +639,6 @@ void VariablesDialog::updateVariables() {
 			iter->setSelected(true);
 		}
 	}
-	l.clear(); l << tr("User variables"); l << "User items";
-	iter = new QTreeWidgetItem(iter3, l);
-	if(selected_category == "User items") {
-		iter->setSelected(true);
-	}
 	if(has_inactive) {
 		//add "Inactive" category if there are inactive variables
 		l.clear(); l << tr("Inactive"); l << "Inactive";
@@ -568,12 +647,24 @@ void VariablesDialog::updateVariables() {
 			iter->setSelected(true);
 		}
 	}
+	iter3->setExpanded(true);
 	if(categoriesView->selectedItems().isEmpty()) {
 		//if no category has been selected (previously selected has been renamed/deleted), select "All"
 		selected_category = "All";
-		iter3->setExpanded(true);
 		iter3->setSelected(true);
 	}
+	searchEdit->blockSignals(true);
+	searchEdit->clear();
+	searchEdit->blockSignals(false);
+	variablesModel->setFilter(selected_category);
+	QModelIndex index;
+	if(item_sel) index = variablesModel->mapFromSource(item_sel->index());
+	if(!index.isValid()) index = variablesModel->index(0, 0);
+	if(index.isValid()) {
+		variablesView->selectionModel()->setCurrentIndex(index, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
+		variablesView->scrollTo(index);
+	}
+	selectedVariableChanged(index, QModelIndex());
 }
 void VariablesDialog::setSearch(const QString &str) {
 	searchEdit->setText(str);
